/**************************************************************************
***    
*** Copyright (c) 2004 Regents of the University of Michigan,
***               Hayward H. Chan and Igor L. Markov
***
***  Contact author(s): hhchan@umich.edu, imarkov@umich.edu
***  Original Affiliation:   EECS Department, 
***                          The University of Michigan,
***                          Ann Arbor, MI 48109-2122
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/
#include "mainhiersoftst.h"
#include "interfaceutil.h"

#include "datastrbacksoftst.h"
#include "datastrfrontsoftst.h"
#include "datastrhiersoftst.h"
#include "enginesoftst.h"
#include "enginehiersoftst.h"

// --------------------------------------------------------
void main_hier(int argc, char *argv[])
{   
   ifstream infile;
   ofstream outfile;

   cout << "---< HIERARCHICAL Packing >---" << endl;
   infile.open(argv[1]);
   if (!infile.good())
   {
      cout << "ERROR: cannot open file: " << argv[1] << endl;
      exit(1);
   }

   outfile.open(argv[2]);
   if (!outfile.good())
   {
      cout << "ERROR: cannot open file: " << argv[3] << endl;
      exit(1);
   }

   int format = -1;
   if (!strcmp(argv[3], "--txt"))
      format = BlockInfoType::TXT;
   else if (!strcmp(argv[3], "--soft"))
      format = BlockInfoType::SOFT;
   else
   {
      cout << "ERROR: invalid input format \"" << argv[3]
           << "\"." << endl;
      exit(1);
   }

   // -----set default values-----
   if (format == BlockInfoType::TXT)
      HIER_ACCURACY = DEFAULT_HIER_TXT_ACCURACY;
   else if (format == BlockInfoType::SOFT)
      HIER_ACCURACY = DEFAULT_HIER_SOFT_ACCURACY;
   
   // -----parse optional options-----
   for (int i = 4; i < argc; i++)
      if (!strcmp(argv[i], "--fixed"))
      {
         if (format != BlockInfoType::TXT)
         {
            cout << "ERROR: \"--fixed\" must be used with \"--txt\"."
                 << endl;
            exit(1);
         }
         else
            format = BlockInfoType::TXT_FIXED;
      }
      else if (!strcmp(argv[i], "--HIER_CLUSTER_BASE"))
      {
         i++;
         SetInt(argc, argv, i,
                SoftHierEngineType::HIER_CLUSTER_BASE);
      }
      else if (!strcmp(argv[i], "--HIER_CLUSTER_AREA_DEV"))
      {
         i++;
         SetDouble(argc, argv, i,
                   SoftHierEngineType::HIER_CLUSTER_AREA_DEV);
      }
      else if (!strcmp(argv[i], "--HIER_INIT_AR"))
      {
         i++;
         SetDouble(argc, argv, i,
                   SoftHierEngineType::HIER_AR);
      }
      else if (!strcmp(argv[i], "--HIER_WIDTH_INCRE"))
      {
         i++;
         SetDouble(argc, argv, i,
                   SoftHierEngineType::HIER_WIDTH_INCRE);
      }
      else if (!strcmp(argv[i], "--HIER_HEIGHT_INCRE"))
      {
         i++;
         SetDouble(argc, argv, i,
                   SoftHierEngineType::HIER_HEIGHT_INCRE);
      }
      else if (!strcmp(argv[i], "--HIER_USE_AR_LEVEL"))
      {
         i++;
         SetInt(argc, argv, i, 
                SoftHierEngineType::HIER_USE_AR_LEVEL);
      }
      else if (!strcmp(argv[i], "--HIER_INIT_DEADSPACE"))
      {
         i++;
         SetDouble(argc, argv, i,
                   SoftHierEngineType::HIER_INIT_DEADSPACE);
         SoftHierEngineType::HIER_INIT_DEADSPACE /= 100;
      }
      else if (!strcmp(argv[i], "--HIER_DEADSPACE_INCRE"))
      {
         i++;
         SetDouble(argc, argv, i,
                   SoftHierEngineType::HIER_DEADSPACE_INCRE);
      }
      else if (!strcmp(argv[i], "--HIER_SIMILARITY_THRESHOLD"))
      {
         i++;
         SetDouble(argc, argv, i,
                   SoftHierEngineType::HIER_SIMILARITY_THRESHOLD);
      }
      else if (!strcmp(argv[i], "--AREA_ACCURACY"))
      {
         i++;
         SetDouble(argc, argv, i, HIER_ACCURACY);
         HIER_ACCURACY /= 100;
         HIER_ACCURACY += 1;
      }
      else if (!strcmp(argv[i], "--CURVE_ACCURACY"))
      {
         i++;
         SetInt(argc, argv, i, BlockInfoType::CURVE_ACCURACY);
      }
      else if (!strcmp(argv[i], "-h") ||
               !strcmp(argv[i], "--hierarchical"))
      {}
      else
      {
         cout << "ERROR: Invalid option: " << argv[i] << endl;
         exit(1);
      }

   // -----output parameters-----
   cout.setf(ios::fixed);
   cout.precision(2);
   cout << "Details: " << endl;
   cout << "  - Block type: ";
   if (format == BlockInfoType::TXT)
      cout << "all hard, free orientations." << endl;
   else if (format == BlockInfoType::TXT_FIXED)
      cout << "all hard, fixed orientations." << endl;
   else if (format == BlockInfoType::SOFT)
   {
      cout << "all soft ";
      cout << "(Curve Accuracy = " << BlockInfoType::CURVE_ACCURACY
           << ")." << endl;
   }
   cout << "  - Area accuracy: " << ((HIER_ACCURACY-1) * 100)
        << "%." << endl;
      
   cout << "Parameters:" << endl;
   cout << "  - HIER_CLUSTER_BASE: "
        << SoftHierEngineType::HIER_CLUSTER_BASE << endl;
   cout << "  - HIER_CLUSTER_AREA_DEV: "
        << SoftHierEngineType::HIER_CLUSTER_AREA_DEV << endl;
   cout << "  - HIER_INIT_AR: "
        << SoftHierEngineType::HIER_AR << endl;
   cout << "  - HIER_USE_AR_LEVEL: "
        << SoftHierEngineType::HIER_USE_AR_LEVEL;
   if (SoftHierEngineType::HIER_USE_AR_LEVEL < 0)
      cout << " (i.e. aspect ratio constraint not imposed.)";
   cout << endl;
   cout << "  - HIER_WIDTH_INCRE: "
        << SoftHierEngineType::HIER_WIDTH_INCRE << endl;
   cout << "  - HIER_HEIGHT_INCRE: "
        << SoftHierEngineType::HIER_HEIGHT_INCRE << endl;
   cout << "  - HIER_INIT_DEADSPACE: "
        << SoftHierEngineType::HIER_INIT_DEADSPACE << endl;
   cout << "  - HIER_DEADSPACE_INCRE: "
        << SoftHierEngineType::HIER_DEADSPACE_INCRE << endl;
   cout << "  - HIER_SIMILARITY_THRESHOLD: "
        << SoftHierEngineType::HIER_SIMILARITY_THRESHOLD << endl;
   cout << endl;
   
   // -----Core program starts here-----
   BlockInfoType blockinfo(format, infile);
   SoftHierEngineType engine(blockinfo);

   SoftSliceRecord *ssrPtr = engine(format);

   SoftPacking final_packing(*ssrPtr, blockinfo);
   final_packing.output(outfile);

   double final_blkArea = final_packing.blockArea;
   double final_deadspace = final_packing.deadspace;
   double final_totalArea = final_blkArea + final_deadspace;
   cout << endl;
   cout << "After operator optimization," << endl;
   cout << "blkArea: " << setw(11) << final_blkArea
        << " deadspace: " << setw(11) << final_deadspace
        << " (" << ((final_deadspace / final_blkArea) * 100)
        << "%) time: " << getTotalTime() << endl;

   cout << endl;
   cout << "total area: " << setw(11)
        << final_totalArea << endl;
   cout << "block area: " << setw(11)
        << final_blkArea << endl;
   cout << "deadspace:  " << setw(11)
        << final_deadspace << endl << endl;

   if (outfile.good())
      cout << "Output successfully written to " << argv[2] << endl;
   else
      cout << "Something wrong with the file " << argv[2] << endl;

   outfile.close();
   delete ssrPtr;
}
// --------------------------------------------------------
